<?php
// +----------------------------------------------------------------------+
// | Copyright 2005-2009 the Seasar Foundation and the Others.            |
// +----------------------------------------------------------------------+
// | Licensed under the Apache License, Version 2.0 (the "License");      |
// | you may not use this file except in compliance with the License.     |
// | You may obtain a copy of the License at                              |
// |                                                                      |
// |     http://www.apache.org/licenses/LICENSE-2.0                       |
// |                                                                      |
// | Unless required by applicable law or agreed to in writing, software  |
// | distributed under the License is distributed on an "AS IS" BASIS,    |
// | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,                        |
// | either express or implied. See the License for the specific language |
// | governing permissions and limitations under the License.             |
// +----------------------------------------------------------------------+
/**
 * アノテーションを読み込むユーティリティクラスです。
 *
 * @copyright 2005-2009 the Seasar Foundation and the Others.
 * @license   http://www.apache.org/licenses/LICENSE-2.0
 * @link      http://s2container.php5.seasar.org/
 * @version   SVN: $Id:$
 * @since     Class available since Release 2.0.0
 * @package   seasar.util
 * @author    klove
 */
namespace seasar\util;
class Annotation {

    /**
     * @var boolean
     */
    public static $CONSTANT = true;

    /**
     * @var boolean
     */
    public static $COMMENT  = true;

    /**
     * @var array
     */
    private static $spool = array();

    /**
     * @var array
     */
    private static $formattedComments = array();

    /**
     * 定数アノテーションを読み込むかどうかを設定します。
     *
     * @param boolean $val
     */
    public static function readConstantAnnotation($val = true) {
        self::$CONSTANT = $val;
    }

    /**
     * コメントアノテーションを読み込むかどうかを設定します。
     *
     * @param boolean $val
     */
    public static function readCommentAnnotation($val = true) {
        self::$COMMENT = $val;
    }

    /**
     * アノテーションデータをスプールに登録します。
     *
     * @param string $annoKey
     * @param array  $annoArray
     * @return array
     */
    public static function put($annoKey, array $annoArray) {
        self::$spool[$annoKey] = $annoArray;
    }

    /**
     * アノテーションを取得します。
     *
     * @param \ReflectionClass|ReflectionMethod|ReflectionProperty $reflection
     * @param string $annotation
     * @return array
     */
    public static function get($reflection, $annotation) {
        if (self::$CONSTANT and self::hasConstantAnnotation($reflection, $annotation)) {
            return self::getConstantAnnotation($reflection, $annotation);
        }
        if (self::$COMMENT) {
            return self::getCommentAnnotation($reflection, $annotation);
        }
        throw new \seasar\exception\AnnotationNotFoundException($reflection, $annotation);
    }

    /**
     * 定数アノテーションを取得します。
     *
     * @param \ReflectionClass|ReflectionMethod|ReflectionProperty $reflection
     * @param string $annotation
     * @return array
     */
    public static function getConstantAnnotation($reflection, $key) {
        list($annotation, $atAnnotation) = self::constructAnnotationKey($key);
        self::validateReflection($reflection);
        $annoKey = self::constructKey($reflection) . '_' . $annotation;
        if (isset(self::$spool[$annoKey])) {
            return self::$spool[$annoKey];
        }

        if ($reflection instanceof \ReflectionProperty or
            $reflection instanceof \ReflectionMethod) {
            $annotation = $reflection->getName() . '_' . $annotation;
            $reflection = $reflection->getDeclaringClass();
        }

        $annoArray = null;
        if ($reflection->hasConstant($annotation)) {
            $value = EvalUtil::formatArrayExpression(trim($reflection->getConstant($annotation)));
            $annoArray = EvalUtil::execute($value);
        }

        if (is_array($annoArray)) {
            self::$spool[$annoKey] = $annoArray;
        } else {
            throw new \seasar\exception\AnnotationNotFoundException($reflection, $annotation);
        }
        return self::$spool[$annoKey];
    }

    /**
     * コメントアノテーションを取得します。
     * コメントアノテーションのフォーマットは、次の2つなります。
     *   (アノテーション名)(0個以上の空白)(PHP配列フォーマット値);
     *   (アノテーション名)(0個以上の空白)(PHP配列フォーマット値)(スペース)
     *   サンプル
     *     @Hoge(1, 2, 3);                 => array(1, 2, 3)
     *     @Huga ('a' => 1, 'b' => 2);     => array('a' => 1, 'b' => 2)
     *     @Foo(100)                       => array(100)
     *     @Bar ('a' => 1, 'b' => 2)       => array('a' => 1, 'b' => 2)
     *  次のケースでは、パースに失敗します。
     *    @Hoge("new Pdo() ")
     *    最後尾にセミコロンがないため、先頭から最短マッチで「) 」までが処置されます。
     *    閉じ括弧の後ろのスペースを削除するか、最後尾にセミコロンを付ける必要があります。
     *
     * @param \ReflectionClass|ReflectionMethod|ReflectionProperty $reflection
     * @param string $annotation
     * @return array
     */
    public static function getCommentAnnotation($reflection, $key) {
        list($annotation, $atAnnotation) = self::constructAnnotationKey($key);
        self::validateReflection($reflection);
        $refKey  = self::constructKey($reflection);
        $annoKey = $refKey . '_' . $annotation;
        if (isset(self::$spool[$annoKey])) {
            return self::$spool[$annoKey];
        }

        if (!isset(self::$formattedComments[$refKey])) {
            self::$formattedComments[$refKey] = self::formatCommentLine($reflection->getDocComment());
        }
        $comment = self::$formattedComments[$refKey];
        $annoArray = null;
        $matches = array();
        if (preg_match("/$atAnnotation\s*\((.*?)\);/iu", $comment, $matches) or
            preg_match("/$atAnnotation\s*\((.*?)\)\s/iu", $comment, $matches)) {
            $value = EvalUtil::formatArrayExpression(trim($matches[1]));
            $annoArray = EvalUtil::execute($value);
        }

        if (is_array($annoArray)) {
            self::$spool[$annoKey] = $annoArray;
        } else {
            throw new \seasar\exception\AnnotationNotFoundException($reflection, $annotation);
        }
        return self::$spool[$annoKey];
    }

    /**
     * アノテーションが存在するかどうかを返します。
     *
     * @param \ReflectionClass|ReflectionMethod|ReflectionProperty $reflection
     * @param string $annotation
     * @return boolean
     */
    public static function has($reflection, $annotation) {
        if (self::$CONSTANT) {
            $ret = self::hasConstantAnnotation($reflection, $annotation);
            if ($ret === true) {
                return true;
            } else if ($ret === false and self::$COMMENT === false) {
                return false;
            }
        }
        if (self::$COMMENT) {
            return self::hasCommentAnnotation($reflection, $annotation);
        }
        return false;
    }

    /**
     * 定数アノテーションが存在するかどうかを返します。
     *
     * @param \ReflectionClass|ReflectionMethod|ReflectionProperty $reflection
     * @param string $annotation
     * @return boolean
     */
    public static function hasConstantAnnotation($reflection, $annotation) {
        try {
            self::getConstantAnnotation($reflection, $annotation);
            return true;
        } catch (\seasar\exception\AnnotationNotFoundException $e) {
            return false;
        }
    }

    /**
     * コメントアノテーションが存在するかどうかを返します。
     *
     * @param \ReflectionClass|ReflectionMethod|ReflectionProperty $reflection
     * @param string $annotation
     * @return boolean
     */
    public static function hasCommentAnnotation($reflection, $annotation) {
        try {
            self::getCommentAnnotation($reflection, $annotation);
            return true;
        } catch (\seasar\exception\AnnotationNotFoundException $e) {
            return false;
        }
    }

    /**
     * コメントをフォーマットします。
     *
     * @param string $commentLine
     * @return string
     */
    public static function formatCommentLine($commentLine) {
        $comments = preg_split('/[\n\r]/', $commentLine, -1, PREG_SPLIT_NO_EMPTY);
        $comment = ' ';
        $lineCount = count($comments);
        if ($lineCount == 0) {
            return '';
        } else if ($lineCount == 1) {
            $comment = substr(trim($comments[0]), 3, -2);
        } else {
            $atFound = false;
            $comment .= substr(trim($comments[0]), 3) . ' ';
            if (0 < strpos($comment, '@')) {
                $atFound = true;
            }
            for($i=1; $i<$lineCount-1; $i++) {
                if ($atFound === false) {
                    if (0 < strpos($comments[$i], '@')) {
                        $atFound = true;
                    }
                }
                if ($atFound === true) {
                    $comment .= substr(trim($comments[$i]), 1) . ' ';
                }
            }
            $last = substr(trim($comments[$lineCount-1]), 0, -2);
            if (0 < strlen($last)) {
                $comment .= substr($last, 1);
            }
        }
        return $comment;
    }

    /**
     * spoolに保存するための一意のキーを作成します。
     *
     * @param \ReflectionClass|ReflectionMethod|ReflectionProperty $reflection
     * @param string $annotation
     * @return string
     */
    public static function constructKey($reflection) {
        if ($reflection instanceof \ReflectionClass) {
            return $reflection->getName() . '::CLASS';
        } else if ($reflection instanceof \ReflectionMethod) {
            return $reflection->getDeclaringClass()->getName() . '::' . $reflection->getName() . '::METHOD';
        } else {
            return $reflection->getDeclaringClass()->getName() . '::' . $reflection->getName() . '::PROPERTY';
        }
    }

    /**
     * 対応しているReflectionクラスかどうかを検証します。
     *
     * @param mixed
     * @throw \seasar\exception\AnnotationNotSupportedException
     */
    protected static function validateReflection($reflection) {
        if (!$reflection instanceof \ReflectionClass and
            !$reflection instanceof \ReflectionMethod and
            !$reflection instanceof \ReflectionProperty) {
            throw new \seasar\exception\AnnotationNotSupportedException('unsupported reflection.');
        }
    }

    /**
     * 指定されたアノテーションキーから、@マークが付いているキーと付いていないキーを作成します。
     *
     * @param string $key
     * @return array @無しキー、@付きキー
     */
    public static function constructAnnotationKey($key) {
        if (strpos($key, '@') === 0) {
            return array(substr($key, 1), $key);
        } else {
            return array($key, '@' . $key);
        }
    }
}

